# pedestal.tcl Tue Jul  4 12:47:56 BST 1995
# calculates pedestal values for ADCs in LeCroy4300B
# expects histograms from no input to ADC
# reads spectrum, finds peak, writes peak channel number+constant to register
# cycles round all possible vsns (modules) and each SubAddress in them (adc)
# uses fdt32 lookup table file to find group&item number for each vsn:sa
# uses group&item to generate spectrum name

#    Updated Sept/Oct 1997 for Euroball


set framename PD
set frametitle "LeCroy 4300B Pedestal Calculation"
set icontitle PD

set debug 0
set started 0

# The lowest possible group number for Diamant
set diamant 81
set itemnames {Energy ID TAC}

# a constant value to be added in
set const 15

# These are 1024 channel spectra
set lowchan 0
set highchan 240 
# The peakfind source comments say that a acceptance > 100(%) means all peaks 
set all 101
# 5 channels seems to be a common default for FWHM
set FWHM 5

#set vsnmax 255
set vsnmax 16
set LUT(0:0) 0

set resource camac

set LUTdir $env(EXPERIMENTBASE)/FVI
set LUTfile LUT.current

proc set-experiment dir {
   global env LUTdir
   set LUTdir "$env(EXPERIMENTBASE)/FVI"
}

proc write-reg {r v} {
   set z [catch {eg "(register :name '$r'  :value '$v')"} m]
   if {$z!=0} then {error "error $m writing $r"}
   }

proc round x {scan [expr $x+0.5] %d n; return $n}

proc readLUTfile {} {
  global LUT
  global LUTdir LUTfile 

   set z [catch {open $LUTdir/$LUTfile r} file]
   if {$z != 0} then {set-footer "Could not open $LUTdir/$LUTfile - $file"; return 1}
   set l [read $file]
   close $file
  
#    initialise LUT array with a default value
   for {set vsn 0} {$vsn < 256} {incr vsn} {
      for {set subaddr 0} {$subaddr < 16} {incr subaddr} {
        set LUT($vsn:$subaddr) 0
      }
   }

   foreach line [split $l \n] {
     set element [split [first $line] +]
     if {[string range [first $element] 0 2] != "lut"} then {continue}
     set lutoffset [second $element]
     set lutoffset [expr $lutoffset / 2]
     set vsn [expr $lutoffset / 16]
     set subaddr [expr $lutoffset % 16]
     set item [second $line]
     set group [third $line]
     set LUT($vsn:$subaddr) "$group:$item"
   }
   return 0
}

proc parse-peaks-for-first {stuff} {
   global debug
   if {$debug} then {
      puts stdout "parsing $stuff"
   }
   set lines [split $stuff "\n"]
   set ll {}
   foreach line [lreplace $lines 0 1] {
      if {[scan $line "%d %f(%f) %f" i c j a]==4} then {
         set ll [round $c]
         if {$debug} then {
            puts stdout "found peak at $ll"
         }
#    return with first peak found
         return $ll
      }
   }
   if {$debug} then {
      puts stdout "using peak at $ll"
   }
   return $ll
}

proc parse-peaks-for-largest {stuff} {
# look for largest peak in peakfind output (should use tallest?)
   global debug
   if {$debug} then {
      puts stdout "parsing $stuff"
   }
   set lines [split $stuff "\n"]
# lc will contain a list of channels
# la will contain the corresponding peak heights (areas)
   set lc {}
   set la {}
   foreach line [lreplace $lines 0 1] {
      if {[scan $line "%d %f(%f) %f" i c j a]==4} then {
         lappend lc [round $c]
         lappend la [round $a]
         if {$debug} then {
            puts stdout "found peak at [round $c]"
         }
      }
   }
# look for the largest height from la - remember its index to pick out the
# corresponding channel number from lc
   set m 0
   if {[llength $lc] > 1} then {
      set l [first $la]
      set i 0
      foreach a $la {
         if {$a > $l} then {set l $a; set m $i}
         incr i
      }
   }
   set c [lindex $lc $m]
   if {$debug} then {
      puts stdout "using peak at $c"
   }
   return $c
}

proc parse-peaks {stuff} {
#   parse-peaks-for-first $stuff
   parse-peaks-for-largest $stuff
}

proc calc-pedestals {} {
   global debug resource vsnmax LUT itemnames const
   global lowchan highchan FWHM all diamant
   set p 0
   set adcs 0
   set fl ""
   set grpitem 0
# Loop round all VSN/SA combinations
   for {set vsn 0} {$vsn <= $vsnmax} {incr vsn}  {
      if {$fl=="-"} {set fl ""} else {set fl "-"}
 #     if {!$grpitem} {set-footer $fl}
      for {set sa 0} {$sa < 16} {incr sa} {
# Read LUT to find Group/Item for this VSN/SA
         set grpitem $LUT($vsn:$sa)
         if {$grpitem == 0} then {continue}
         set grp  [first  [split $grpitem :]]
         set item [second [split $grpitem :]]
# found an ADC with a corresponding Group/Item
         incr adcs
# Turn Group/Item into spectrum name
         set s "/hist/Diamant[expr $grp - $diamant].[lindex $itemnames $item]"
         if {$debug} then {puts stdout "spectrum: $s"}
         set-footer "finding peak in $s"
# Look for peaks in histogram
         set z [catch {eval exec peakfind \
            -spectrum [transparent $s] \
            -limit $lowchan $highchan \
            -fwhm $FWHM \
            -accept $all } m]
         if {$z} then {
            insert-log "No histogram for $s (lc4300b#$vsn.adc$sa)"
         } else {
# Find channel of peak
            set c [parse-peaks $m]
            if {$c!=""} then {
               incr p
               insert-log "Histogram $s: pedestal value for $vsn,$sa (VSN,SA) is $c+$const ([expr $c+$const])"
# Write channel number (+ constant) to pedestal register of VSN/SA (module/ADC)
      #         write-reg /$resource/lc4300b#$vsn.adc$sa.pedestal [expr $c+$const]
            } else {
               insert-log "No peak found in $s (lc4300b#$vsn.adc$sa)"
            }
         }
      }
   }
   set-footer "set pedestal values for $p out of $adcs declared ADCs"
}


proc make-frame {} {
  global env vsnmax const

  :frame "[self frame-name]" :label "[self frame-title]" :show-footer T  :resize F \
      :icon-image "@$env(ICONHOME)/default" \
      :icon-name "[self icon-title]"

    :panel PANEL1 :layout T :padx 10 :pady 5
      :layout V; :row-gap 10; :col-gap 10

      :message M1 :label "See the Session Log for errors" :bold T
      :message M2 :label "maximum VSN assumed $vsnmax;  pedestal offset constant $const"

    :show T
}

proc start {} {
   global started lut
   if {$started == 0} then {make-frame}
   set started 1
# if we put the body of doit here or call doit sooner than after 1000ms
# then not everything in pdformat is displayed.
# this is probably due to X-client caching/flushing?
   after 1000
   set-busy
   if {[readLUTfile] == 0} then {calc-pedestals}
   clear-busy
}

